static char rcsid[] = "@(#)$Id: write_rc.c,v 2.13 2016/05/10 18:35:03 hurtta Exp $";

/******************************************************************************
 *  The Elm (ME+) Mail System  -  $Revision: 2.13 $   $State: Exp $
 *
 *  Author: Kari Hurtta <hurtta+elm@siilo.FMI.FI> 
 *                  (was hurtta+elm@posti.FMI.FI, hurtta+elm@ozone.FMI.FI)
 *      or  Kari Hurtta <elm@elmme-mailer.org>
 *****************************************************************************
 * Some code based on Elm 2.4 src/save_opts.c
 * That code was following copyright:
 *
 *  The Elm Mail System 
 *
 *			Copyright (c) 1988-1992 USENET Community Trust
 *			Copyright (c) 1986,1987 Dave Taylor
 *****************************************************************************/

#include "elm_defs.h"
#include "rc_imp.h"
#include "save_opts.h"
#include "s_elmrc.h"

#include <errno.h> 
#ifndef ANSI_C
extern int errno;
#endif

DEBUG_VAR(Debug,__FILE__,"config");

charset_t rcset;             /* What charset use for writing */
static const char * CHARSETOPT = "@charset";

void write_rc_part(targetfile_F,
		   part_tag,commentfile_F,
		   global,rcset,rc_options,rc_option_count)
     FILE       * targetfile_F;
     const char * part_tag          /* NULL for main part */;
     FILE       * commentfile_F     /* NULL if no commentfile */;    
     int          global;
     charset_t    rcset;
     struct rc_save_info_rec * rc_options;
     int                       rc_option_count;
{
    int x;

    if (commentfile_F) {
	char line_buffer[SLEN];
	int x;
	int len;
	int local_value;
	int skip = 1;

	char * section     = NULL;
	int    section_len = 0;

	while (0  < (len = 
		     mail_gets(line_buffer, sizeof line_buffer, 
			       commentfile_F))) {
	    
	    int y = 0;

	    if (line_buffer[len-1] != '\n') {
		DPRINT(Debug,1,(&Debug, 
				"%s commentile: Too long line (read len=%d): %s\n",
				part_tag ? part_tag : "Main",

				len,line_buffer));
		break;
	    }
	    line_buffer[--len] = '\0';

	    if (0 == len)
		continue;

	    if ('#' == line_buffer[0] || whitespace(line_buffer[0])) {
		
		if (strncmp(line_buffer, "#$HDR", 5) == 0) {
		    skip = 0;
		    continue;
		}
		
		if (skip)
		    continue;
		
		/* Copy initial comments */
		
		fprintf(targetfile_F,"%s\n",line_buffer);
		continue;
	    }
	    skip = 1;

	    /* trim trailing space */
	    while (len > 0 && whitespace(line_buffer[len-1]))
		line_buffer[--len] = '\0';
	    if (0 == len)
		continue;

	    /* get section header */ 
	    if ('$' == line_buffer[0]) {

		if (section)
		    free(section);
		section     = NULL;
		section_len = 0;
			

		if (0 == strcmp(line_buffer,"$user")) {
		    /*  User comment */
		    
		    skip = global;
		    continue;
		} else if (0 == strcmp(line_buffer,"$global")) {

		    skip = !global;
		    continue;
		} else if (0 == strcmp(line_buffer,"$usersection")) {

		    if (global)
			continue;
		} else if (0 == strcmp(line_buffer,"$globalsection")) {

		    if (!global)
			continue;
		}

		while (0  < (len = 
			     mail_gets(line_buffer, sizeof line_buffer, 
				       commentfile_F))) {
		    if ('#' != line_buffer[0])
			break;

		    section = safe_realloc(section,section_len+len+1);

		    strfcpy(section+section_len,line_buffer,len+1);
		    section_len += len;

		    continue;
		}

		skip = 1;
		continue;
	    }

	    for (x = 0; x < rc_option_count; x++) {
		y = strcmp(line_buffer, rc_options[x].name);
		if (y <= 0)
		    break;
	    }
	    
	    if (x >= rc_option_count || y != 0) {
		
		DPRINT(Debug,15,(&Debug,
				 "Skipping %s option %s ... \n",
				 part_tag ? part_tag : "main",
				 line_buffer));
		
		continue;
	    }

	    if (global && 
		! (rc_options[x].flags & FL_CHANGED) &&
		! (rc_options[x].flags & FL_SYS) &&
		! (rc_options[x].flags & FL_SHOWSYS)) {
		DPRINT(Debug,15,(&Debug,
				 "Skipping %s user option %s ... \n",
				 part_tag ? part_tag : "main",
				 line_buffer));
		
		continue;
	    }
	    
	    if (!global &&
		(rc_options[x].flags & FL_SYS)) {
		DPRINT(Debug,15,(&Debug,
				 "Skipping %s system option %s ... \n",
				 part_tag ? part_tag : "main",
				 line_buffer));
		
		continue;
	    }
	    
	    if (section) {
		fprintf(targetfile_F,"\n");

		fputs(section,targetfile_F);

		free(section);
		section = NULL;

		section_len = 0;
	    }

	    fprintf(targetfile_F,"\n");

	    /* Copy comments */
	    while (0  < (len = 
			 mail_gets(line_buffer, sizeof line_buffer, 
				   commentfile_F))) {
		if ('#' != line_buffer[0])
		    break;
		fputs(line_buffer,targetfile_F);
	    }
	    
	    if (global)
		local_value = rc_options[x].flags & FL_CHANGED;
	    else
		local_value = rc_options[x].flags & FL_LOCAL;
	    if (RCTYPE_magic != rc_options[x].dt_type->magic)
		panic("RC PANIC",__FILE__,__LINE__,"write_rc",
		      "Bad config item type",0);

	    DPRINT(Debug,15,(&Debug,
			     "Writing %s option %s ... %s\n",
			     part_tag ? part_tag : "main",
			     rc_options[x].name,
			     local_value ? "(local value)" : ""));
	    
	    rc_options[x].dt_type->print_value(targetfile_F,
					       & rc_options[x],
					       !local_value,
					       0 != ( rc_options[x].flags & 
						      FL_ASCIIFIRST)
					       ? ASCII_SET : rcset);
	    	    
	    rc_options[x].flags |= FL_SAVED;

	    /* Copy extra comments */
	    skip = 0;
	}

	if (section)
	    free(section);
	section     = NULL;
	section_len = 0;

    }

    for (x = 0; x < rc_option_count; x++) {
	int local_value;
	
	
	if (rc_options[x].flags & FL_SAVED)
	    continue;
	
	/* This is written to top of file */
	if (0 == strcmp(rc_options[x].name,CHARSETOPT))
	    continue;

	if (global && 
	    ! (rc_options[x].flags & FL_CHANGED) &&
	    ! (rc_options[x].flags & FL_SYS) &&
	    ! (rc_options[x].flags & FL_SHOWSYS)) {
	    DPRINT(Debug,15,(&Debug,
			     "Skipping %s user option %s ... \n",
			     part_tag ? part_tag : "main",
			     rc_options[x].name));
	    
	    continue;
	}
	
	if (!global &&
	    (rc_options[x].flags & FL_SYS)) {
	    DPRINT(Debug,15,(&Debug,
			     "Skipping system option %s ... \n",
			     rc_options[x].name));
	    
	    continue;
	}
	
	fprintf(targetfile_F,"\n");
	
	if (global)
	    local_value = rc_options[x].flags & FL_CHANGED;
	else
	    local_value = rc_options[x].flags & FL_LOCAL;
	if (RCTYPE_magic != rc_options[x].dt_type->magic)
	    panic("RC PANIC",__FILE__,__LINE__,"write_rc_part",
		  "Bad config item type",0);

	DPRINT(Debug,15,(&Debug,
			 "Writing %s option %s ... %s (no comment)\n",
			 part_tag ? part_tag : "main",
			 rc_options[x].name,
			 local_value ? "(local value)" : ""));
	
	
	rc_options[x].dt_type->print_value(targetfile_F,
					  & rc_options[x],
					  !local_value,
					  0 != ( rc_options[x].flags & FL_ASCIIFIRST)
					  ? ASCII_SET : rcset);
    }
}

int write_rc(targetfile,commentfile,global,actor,username,version_buff)
     char *targetfile; 
     FILE * commentfile; 
     int global;
     const char *actor;
     struct string *username;
     char *version_buff;
{
    int close_comment_file = 0;
    char *tmp  = NULL;
    int res = 0;
    int errcode;

    const char * sysnam = NULL;
    int default_commentfile = 0;  /* Use default commentile  */

    FILE *f = NULL;

    enum editor_propline_v propline_mode = 
	give_dt_enumerate_as_int(&editor_elmrc_propline);

    if (!rcset) {
	DPRINT(Debug,10,(&Debug, 
			 "write_rc: rcset is not set, using system charset\n"));

	rcset = system_charset;
    }

    sysnam = get_charset_MIME_name(rcset);

    if (!commentfile) {
	commentfile = fopen(ELMRC_INFO,"r");

	if (!commentfile) {
	    int err = errno;
	
	    if (ENOENT != err) 
		lib_error(CATGETS(elm_msg_cat, ElmrcSet, 
				  ElmrcCommentFileError,
				  "Comment file %s not readable: %s\n"),
			  ELMRC_INFO,strerror(err));

	    lib_error(CATGETS(elm_msg_cat, ElmrcSet, 
			      ElmrcSavingWithoutComments,
			      "Warning: saving without comments! Can't get to %s."),
		      ELMRC_INFO);
	    
	    DPRINT(Debug,8,(&Debug,
			    "no comment file %s: %s\n",
			    ELMRC_INFO,
			    strerror(err)));
	} else 
	    close_comment_file++;

	default_commentfile = 1;
    }


    tmp = elm_message(FRM("%s.N"),targetfile);
    
    errcode = can_open(tmp,"w");

    if (errcode) {
	lib_error(CATGETS(elm_msg_cat, ElmrcSet, ElmrcFileNotWriteable,
			  "File %s is not writeable: %s"),
		  tmp, strerror(errcode));
	
	res = 0;
	goto fail;
    }


    f = fopen(tmp,"w");
    if (!f) {
	int errcode = errno;
	lib_error(CATGETS(elm_msg_cat, ElmrcSet, ElmrcFileNotWriteable,
			  "File %s is not writeable: %s"),
		  tmp, strerror(errcode));
	
	res = 0;
	goto fail;
    }
    
    /* Must have first line */
    if (propline_mode != editor_propline_ignore)
	write_editor_propline(current_editor_propline,propline_mode,
			      f,"# ","\n",rcset);

    if (!global)
	elm_fprintf(f, 
		    CATGETS(elm_msg_cat, ElmrcSet, ElmrcOptionsFile,
			    "#\n# .elm/elmrc - options file for the ELM mail system\n#\n"));

    if (sysnam) {
	/* Store used character set */
	
	elm_fprintf(f, 
		    CATGETS(elm_msg_cat, ElmrcSet, ElmrcCharsetExplain,
			    "# Character set used on this file\n"));

	fprintf(f,"%s = %s\n",CHARSETOPT,sysnam);
    }


    if (username) {
	struct string * S =
	    format_string(CATGETS(elm_msg_cat, ElmrcSet, ElmrcSavedAutoFor1X,
				  "# Saved automatically by %s %s for %S\n#\n\n"),
			  actor, version_buff, username);
	char * buffer = NULL;
	int buffer_len = 0;
	int ok UNUSED_VAROK = 
	    sconvert_to_rcset(S,NULL,rcset,&buffer,&buffer_len);

	fwrite(buffer,1,buffer_len,f);

	free(buffer);
	free_string(&S);

    } else
	elm_fprintf(f, 
		    CATGETS(elm_msg_cat, ElmrcSet, ElmrcSavedAutoWrite1,
			    "# Saved automatically by %s %s\n#\n\n"), 
		    actor,version_buff);

    write_rc_part(f,NULL,commentfile,
		  global,rcset,
		  save_info,NUMBER_OF_SAVEABLE_OPTIONS);
			     
#ifdef USE_DLOPEN
    print_local_shared_options(f,global,rcset,
			       default_commentfile);
#endif
    
    if (EOF == fclose(f)) {
	int errcode = errno;
	lib_error(CATGETS(elm_msg_cat, ElmrcSet, ElmrcFileNotWriteable,
			  "File %s is not writeable: %s"),
		  tmp, strerror(errcode));
	
	res = 0;
	goto fail;
    }
    
    if (0 != rename(tmp,targetfile)) {
	int errcode = errno;
	lib_error(CATGETS(elm_msg_cat, ElmrcSet, ElmrcFileNotRenamed,
			  "Failed to rename temporary file to %s: %s"),
		  targetfile, strerror(errcode));
	
	res = 0;
	goto fail;
    }
        
    res = 1;

 fail:
    if (tmp)
	free(tmp);

    if (close_comment_file)
	fclose(commentfile);

    return res;
}


/*
 * Local Variables:
 *  mode:c
 *  c-basic-offset:4
 *  buffer-file-coding-system: iso-8859-1
 * End:
 */

